@@ -1,11 +1,19 @@
 /* decode.c - ber input decoding routines */
-/* $OpenLDAP: pkg/ldap/libraries/liblber/decode.c,v 1.34.6.10 2002/01/04 20:38:18 kurt Exp $ */
-/*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/libraries/liblber/decode.c,v 1.105.2.4 2008/02/11 23:26:41 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1998-2008 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
-/* Portions
- * Copyright (c) 1990 Regents of the University of Michigan.
+/* Portions Copyright (c) 1990 Regents of the University of Michigan.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms are permitted
@@ -15,6 +23,10 @@
  * software without specific prior written permission. This software
  * is provided ``as is'' without express or implied warranty.
  */
+/* ACKNOWLEDGEMENTS:
+ * This work was originally developed by the University of Michigan
+ * (as part of U-MICH LDAP).
+ */
 
 #include "portable.h"
 
@@ -33,6 +45,52 @@ static ber_len_t ber_getnint LDAP_P((
 	ber_int_t *num,
 	ber_len_t len ));
 
+/* out->bv_len should be the buffer size on input */
+int
+ber_decode_oid( BerValue *in, BerValue *out )
+{
+	const unsigned char *der;
+	unsigned long val;
+	unsigned val1;
+	ber_len_t i;
+	char *ptr;
+
+	assert( in != NULL );
+	assert( out != NULL );
+
+	/* need 4 chars/inbyte + \0 for input={7f 7f 7f...} */
+	if ( !out->bv_val || (out->bv_len+3)/4 <= in->bv_len )
+		return -1;
+
+	ptr = NULL;
+	der = (unsigned char *) in->bv_val;
+	val = 0;
+	for ( i=0; i < in->bv_len; i++ ) {
+		val |= der[i] & 0x7f;
+		if ( !( der[i] & 0x80 )) {
+			if ( ptr == NULL ) {
+				/* Initial "x.y": val=x*40+y, x<=2, y<40 if x=2 */
+				ptr = out->bv_val;
+				val1 = (val < 80 ? val/40 : 2);
+				val -= val1*40;
+				ptr += sprintf( ptr, "%u", val1 );
+			}
+			ptr += sprintf( ptr, ".%lu", val );
+			val = 0;
+		} else if ( val - 1UL < LBER_OID_COMPONENT_MAX >> 7 ) {
+			val <<= 7;
+		} else {
+			/* val would overflow, or is 0 from invalid initial 0x80 octet */
+			return -1;
+		}
+	}
+	if ( ptr == NULL || val != 0 )
+		return -1;
+
+	out->bv_len = ptr - out->bv_val;
+	return 0;
+}
+
 /* return the tag - LBER_DEFAULT returned means trouble */
 ber_tag_t
 ber_get_tag( BerElement *ber )
@@ -42,15 +100,20 @@ ber_get_tag( BerElement *ber )
 	unsigned int	i;
 
 	assert( ber != NULL );
-	assert( BER_VALID( ber ) );
+	assert( LBER_VALID( ber ) );
 
-	if ( ber_read( ber, (char *) &xbyte, 1 ) != 1 ) {
+	if ( ber_pvt_ber_remaining( ber ) < 1 ) {
 		return LBER_DEFAULT;
 	}
 
-	tag = xbyte;
+	if ( ber->ber_ptr == ber->ber_buf ) {
+		tag = *(unsigned char *)ber->ber_ptr;
+	} else {
+		tag = ber->ber_tag;
+	}
+	ber->ber_ptr++;
 
-	if ( (xbyte & LBER_BIG_TAG_MASK) != LBER_BIG_TAG_MASK ) {
+	if ( (tag & LBER_BIG_TAG_MASK) != LBER_BIG_TAG_MASK ) {
 		return tag;
 	}
 
@@ -85,7 +148,7 @@ ber_skip_tag( BerElement *ber, ber_len_t *len )
 
 	assert( ber != NULL );
 	assert( len != NULL );
-	assert( BER_VALID( ber ) );
+	assert( LBER_VALID( ber ) );
 
 	/*
 	 * Any ber element looks like this: tag length contents.
@@ -109,13 +172,14 @@ ber_skip_tag( BerElement *ber, ber_len_t *len )
 
 	/*
 	 * Next, read the length.  The first byte contains the length of
-	 * the length.  If bit 8 is set, the length is the long form,
+	 * the length.	If bit 8 is set, the length is the long form,
 	 * otherwise it's the short form.  We don't allow a length that's
 	 * greater than what we can hold in a ber_len_t.
 	 */
 
-	if ( ber_read( ber, (char *) &lc, 1 ) != 1 )
+	if ( ber_read( ber, (char *) &lc, 1 ) != 1 ) {
 		return LBER_DEFAULT;
+	}
 
 	if ( lc & 0x80U ) {
 		noctets = (lc & 0x7fU);
@@ -124,7 +188,7 @@ ber_skip_tag( BerElement *ber, ber_len_t *len )
 			return LBER_DEFAULT;
 		}
 
-		if( (unsigned) ber_read( ber, netlen, noctets ) != noctets ) {
+		if( (unsigned) ber_read( ber, (char *) netlen, noctets ) != noctets ) {
 			return LBER_DEFAULT;
 		}
 
@@ -138,9 +202,10 @@ ber_skip_tag( BerElement *ber, ber_len_t *len )
 	}
 
 	/* BER element should have enough data left */
-	if( *len > ber_pvt_ber_remaining( ber ) ) {
+	if( *len > (ber_len_t) ber_pvt_ber_remaining( ber ) ) {
 		return LBER_DEFAULT;
 	}
+	ber->ber_tag = *(unsigned char *)ber->ber_ptr;
 
 	return tag;
 }
@@ -156,11 +221,13 @@ ber_peek_tag(
 	 */
 
 	char *save;
-	ber_tag_t	tag;
+	ber_tag_t	tag, old;
 
+	old = ber->ber_tag;
 	save = ber->ber_ptr;
 	tag = ber_skip_tag( ber, len );
 	ber->ber_ptr = save;
+	ber->ber_tag = old;
 
 	return tag;
 }
@@ -175,7 +242,7 @@ ber_getnint(
 
 	assert( ber != NULL );
 	assert( num != NULL );
-	assert( BER_VALID( ber ) );
+	assert( LBER_VALID( ber ) );
 
 	/*
 	 * The tag and length have already been stripped off.  We should
@@ -208,6 +275,7 @@ ber_getnint(
 	} else {
 		*num = 0;
 	}
+	ber->ber_tag = *(unsigned char *)ber->ber_ptr;
 
 	return len;
 }
@@ -221,7 +289,7 @@ ber_get_int(
 	ber_len_t	len;
 
 	assert( ber != NULL );
-	assert( BER_VALID( ber ) );
+	assert( LBER_VALID( ber ) );
 
 	if ( (tag = ber_skip_tag( ber, &len )) == LBER_DEFAULT ) {
 		return LBER_DEFAULT;
@@ -252,7 +320,7 @@ ber_get_stringb(
 	ber_tag_t	tag;
 
 	assert( ber != NULL );
-	assert( BER_VALID( ber ) );
+	assert( LBER_VALID( ber ) );
 
 	if ( (tag = ber_skip_tag( ber, &datalen )) == LBER_DEFAULT ) {
 		return LBER_DEFAULT;
@@ -266,6 +334,7 @@ ber_get_stringb(
 	if ( (ber_len_t) ber_read( ber, buf, datalen ) != datalen ) {
 		return LBER_DEFAULT;
 	}
+	ber->ber_tag = *(unsigned char *)ber->ber_ptr;
 
 	buf[datalen] = '\0';
 
@@ -273,32 +342,264 @@ ber_get_stringb(
 	return tag;
 }
 
+/* Definitions for get_string vector
+ *
+ * ChArray, BvArray, and BvVec are self-explanatory.
+ * BvOff is a struct berval embedded in an array of larger structures
+ * of siz bytes at off bytes from the beginning of the struct.
+ */
+enum bgbvc { ChArray, BvArray, BvVec, BvOff };
+
+/* Use this single cookie for state, to keep actual
+ * stack use to the absolute minimum.
+ */
+typedef struct bgbvr {
+	enum bgbvc choice;
+	BerElement *ber;
+	int alloc;
+	ber_len_t siz;
+	ber_len_t off;
+	union {
+		char ***c;
+		BerVarray *ba;
+		struct berval ***bv;
+	} res;
+} bgbvr;
+
+static ber_tag_t
+ber_get_stringbvl( bgbvr *b, ber_len_t *rlen )
+{
+	int i = 0, n;
+	ber_tag_t tag;
+	ber_len_t len;
+	char *last, *orig;
+	struct berval bv, *bvp = NULL;
+
+	/* For rewinding, just like ber_peek_tag() */
+	orig = b->ber->ber_ptr;
+	tag = b->ber->ber_tag;
+
+	if ( ber_first_element( b->ber, &len, &last ) != LBER_DEFAULT ) {
+		for ( ; b->ber->ber_ptr < last; i++ ) {
+			if (ber_skip_tag( b->ber, &len ) == LBER_DEFAULT) break;
+			b->ber->ber_ptr += len;
+			b->ber->ber_tag = *(unsigned char *)b->ber->ber_ptr;
+		}
+	}
+
+	if ( rlen ) *rlen = i;
+
+	if ( i == 0 ) {
+		*b->res.c = NULL;
+		return 0;
+	}
+
+	n = i;
+
+	/* Allocate the result vector */
+	switch (b->choice) {
+	case ChArray:
+		*b->res.c = ber_memalloc_x( (n+1)*sizeof( char * ),
+			b->ber->ber_memctx);
+		if ( *b->res.c == NULL ) return LBER_DEFAULT;
+		(*b->res.c)[n] = NULL;
+		break;
+	case BvArray:
+		*b->res.ba = ber_memalloc_x( (n+1)*sizeof( struct berval ),
+			b->ber->ber_memctx);
+		if ( *b->res.ba == NULL ) return LBER_DEFAULT;
+		(*b->res.ba)[n].bv_val = NULL;
+		break;
+	case BvVec:
+		*b->res.bv = ber_memalloc_x( (n+1)*sizeof( struct berval *),
+			b->ber->ber_memctx);
+		if ( *b->res.bv == NULL ) return LBER_DEFAULT;
+		(*b->res.bv)[n] = NULL;
+		break;
+	case BvOff:
+		*b->res.ba = ber_memalloc_x( (n+1) * b->siz, b->ber->ber_memctx );
+		if ( *b->res.ba == NULL ) return LBER_DEFAULT;
+		((struct berval *)((char *)(*b->res.ba) + n*b->siz +
+			b->off))->bv_val = NULL;
+		break;
+	}
+	b->ber->ber_ptr = orig;
+	b->ber->ber_tag = tag;
+	ber_skip_tag( b->ber, &len );
+	
+	for (n=0; n<i; n++)
+	{
+		tag = ber_next_element( b->ber, &len, last );
+		if ( ber_get_stringbv( b->ber, &bv, b->alloc ) == LBER_DEFAULT ) {
+			goto nomem;
+		}
+
+		/* store my result */
+		switch (b->choice) {
+		case ChArray:
+			(*b->res.c)[n] = bv.bv_val;
+			break;
+		case BvArray:
+			(*b->res.ba)[n] = bv;
+			break;
+		case BvVec:
+			bvp = ber_memalloc_x( sizeof( struct berval ), b->ber->ber_memctx);
+			if ( !bvp ) {
+				LBER_FREE(bv.bv_val);
+				goto nomem;
+			}
+			(*b->res.bv)[n] = bvp;
+			*bvp = bv;
+			break;
+		case BvOff:
+			*(BerVarray)((char *)(*b->res.ba)+n*b->siz+b->off) = bv;
+			break;
+		}
+	}
+	return tag;
+
+nomem:
+	if (b->alloc || b->choice == BvVec) {
+		for (--n; n>=0; n--) {
+			switch(b->choice) {
+			case ChArray:
+				LBER_FREE((*b->res.c)[n]);
+				break;
+			case BvArray:
+				LBER_FREE((*b->res.ba)[n].bv_val);
+				break;
+			case BvVec:
+				LBER_FREE((*b->res.bv)[n]->bv_val);
+				LBER_FREE((*b->res.bv)[n]);
+				break;
+			default:
+				break;
+			}
+		}
+	}
+	LBER_FREE(*b->res.c);
+	*b->res.c = NULL;
+	return LBER_DEFAULT;
+}
+
 ber_tag_t
-ber_get_stringa( BerElement *ber, char **buf )
+ber_get_stringbv( BerElement *ber, struct berval *bv, int option )
 {
-	ber_len_t	datalen;
 	ber_tag_t	tag;
 
 	assert( ber != NULL );
+	assert( bv != NULL );
+
+	assert( LBER_VALID( ber ) );
+
+	if ( (tag = ber_skip_tag( ber, &bv->bv_len )) == LBER_DEFAULT ) {
+		bv->bv_val = NULL;
+		return LBER_DEFAULT;
+	}
+
+	if ( (ber_len_t) ber_pvt_ber_remaining( ber ) < bv->bv_len ) {
+		return LBER_DEFAULT;
+	}
+
+	if ( option & LBER_BV_ALLOC ) {
+		bv->bv_val = (char *) ber_memalloc_x( bv->bv_len + 1,
+			ber->ber_memctx );
+		if ( bv->bv_val == NULL ) {
+			return LBER_DEFAULT;
+		}
+
+		if ( bv->bv_len > 0 && (ber_len_t) ber_read( ber, bv->bv_val,
+			bv->bv_len ) != bv->bv_len )
+		{
+			LBER_FREE( bv->bv_val );
+			bv->bv_val = NULL;
+			return LBER_DEFAULT;
+		}
+	} else {
+		bv->bv_val = ber->ber_ptr;
+		ber->ber_ptr += bv->bv_len;
+	}
+	ber->ber_tag = *(unsigned char *)ber->ber_ptr;
+	if ( !( option & LBER_BV_NOTERM ))
+		bv->bv_val[bv->bv_len] = '\0';
+
+	return tag;
+}
+
+ber_tag_t
+ber_get_stringbv_null( BerElement *ber, struct berval *bv, int option )
+{
+	ber_tag_t	tag;
+
+	assert( ber != NULL );
+	assert( bv != NULL );
+
+	assert( LBER_VALID( ber ) );
+
+	if ( (tag = ber_skip_tag( ber, &bv->bv_len )) == LBER_DEFAULT ) {
+		bv->bv_val = NULL;
+		return LBER_DEFAULT;
+	}
+
+	if ( (ber_len_t) ber_pvt_ber_remaining( ber ) < bv->bv_len ) {
+		return LBER_DEFAULT;
+	}
+
+	if ( bv->bv_len == 0 ) {
+		bv->bv_val = NULL;
+		ber->ber_tag = *(unsigned char *)ber->ber_ptr;
+		return tag;
+	}
+
+	if ( option & LBER_BV_ALLOC ) {
+		bv->bv_val = (char *) ber_memalloc_x( bv->bv_len + 1,
+			ber->ber_memctx );
+		if ( bv->bv_val == NULL ) {
+			return LBER_DEFAULT;
+		}
+
+		if ( bv->bv_len > 0 && (ber_len_t) ber_read( ber, bv->bv_val,
+			bv->bv_len ) != bv->bv_len )
+		{
+			LBER_FREE( bv->bv_val );
+			bv->bv_val = NULL;
+			return LBER_DEFAULT;
+		}
+	} else {
+		bv->bv_val = ber->ber_ptr;
+		ber->ber_ptr += bv->bv_len;
+	}
+	ber->ber_tag = *(unsigned char *)ber->ber_ptr;
+	if ( !( option & LBER_BV_NOTERM ))
+		bv->bv_val[bv->bv_len] = '\0';
+
+	return tag;
+}
+
+ber_tag_t
+ber_get_stringa( BerElement *ber, char **buf )
+{
+	BerValue	bv;
+	ber_tag_t	tag;
+
 	assert( buf != NULL );
 
-	assert( BER_VALID( ber ) );
+	tag = ber_get_stringbv( ber, &bv, LBER_BV_ALLOC );
+	*buf = bv.bv_val;
 
-	if ( (tag = ber_skip_tag( ber, &datalen )) == LBER_DEFAULT ) {
-		*buf = NULL;
-		return LBER_DEFAULT;
-	}
+	return tag;
+}
 
-	if ( (*buf = (char *) LBER_MALLOC( datalen + 1 )) == NULL ) {
-		return LBER_DEFAULT;
-	}
+ber_tag_t
+ber_get_stringa_null( BerElement *ber, char **buf )
+{
+	BerValue	bv;
+	ber_tag_t	tag;
 
-	if ( (ber_len_t) ber_read( ber, *buf, datalen ) != datalen ) {
-		LBER_FREE( *buf );
-		*buf = NULL;
-		return LBER_DEFAULT;
-	}
-	(*buf)[datalen] = '\0';
+	assert( buf != NULL );
+
+	tag = ber_get_stringbv_null( ber, &bv, LBER_BV_ALLOC );
+	*buf = bv.bv_val;
 
 	return tag;
 }
@@ -306,46 +607,22 @@ ber_get_stringa( BerElement *ber, char **buf )
 ber_tag_t
 ber_get_stringal( BerElement *ber, struct berval **bv )
 {
-	ber_len_t	len;
 	ber_tag_t	tag;
 
 	assert( ber != NULL );
 	assert( bv != NULL );
 
-	assert( BER_VALID( ber ) );
-
-	if ( (tag = ber_skip_tag( ber, &len )) == LBER_DEFAULT ) {
-		*bv = NULL;
-		return LBER_DEFAULT;
-	}
-
-	*bv = (struct berval *) LBER_MALLOC( sizeof(struct berval) );
+	*bv = (struct berval *) ber_memalloc_x( sizeof(struct berval),
+		ber->ber_memctx );
 	if ( *bv == NULL ) {
 		return LBER_DEFAULT;
 	}
 
-	if( len == 0 ) {
-		(*bv)->bv_val = NULL;
-		(*bv)->bv_len = 0;
-		return tag;
-	}
-
-	(*bv)->bv_val = (char *) LBER_MALLOC( len + 1 );
-	if ( (*bv)->bv_val == NULL ) {
+	tag = ber_get_stringbv( ber, *bv, LBER_BV_ALLOC );
+	if ( tag == LBER_DEFAULT ) {
 		LBER_FREE( *bv );
 		*bv = NULL;
-		return LBER_DEFAULT;
 	}
-
-	if ( (ber_len_t) ber_read( ber, (*bv)->bv_val, len ) != len ) {
-		ber_bvfree( *bv );
-		*bv = NULL;
-		return LBER_DEFAULT;
-	}
-
-	((*bv)->bv_val)[len] = '\0';
-	(*bv)->bv_len = len;
-
 	return tag;
 }
 
@@ -363,7 +640,7 @@ ber_get_bitstringa(
 	assert( buf != NULL );
 	assert( blen != NULL );
 
-	assert( BER_VALID( ber ) );
+	assert( LBER_VALID( ber ) );
 
 	if ( (tag = ber_skip_tag( ber, &datalen )) == LBER_DEFAULT ) {
 		*buf = NULL;
@@ -371,7 +648,8 @@ ber_get_bitstringa(
 	}
 	--datalen;
 
-	if ( (*buf = (char *) LBER_MALLOC( datalen )) == NULL ) {
+	*buf = (char *) ber_memalloc_x( datalen, ber->ber_memctx );
+	if ( *buf == NULL ) {
 		return LBER_DEFAULT;
 	}
 
@@ -386,6 +664,7 @@ ber_get_bitstringa(
 		*buf = NULL;
 		return LBER_DEFAULT;
 	}
+	ber->ber_tag = *(unsigned char *)ber->ber_ptr;
 
 	*blen = datalen * 8 - unusedbits;
 	return tag;
@@ -398,7 +677,7 @@ ber_get_null( BerElement *ber )
 	ber_tag_t	tag;
 
 	assert( ber != NULL );
-	assert( BER_VALID( ber ) );
+	assert( LBER_VALID( ber ) );
 
 	if ( (tag = ber_skip_tag( ber, &len )) == LBER_DEFAULT ) {
 		return LBER_DEFAULT;
@@ -407,6 +686,7 @@ ber_get_null( BerElement *ber )
 	if ( len != 0 ) {
 		return LBER_DEFAULT;
 	}
+	ber->ber_tag = *(unsigned char *)ber->ber_ptr;
 
 	return( tag );
 }
@@ -422,7 +702,7 @@ ber_get_boolean(
 	assert( ber != NULL );
 	assert( boolval != NULL );
 
-	assert( BER_VALID( ber ) );
+	assert( LBER_VALID( ber ) );
 
 	rc = ber_get_int( ber, &longbool );
 	*boolval = longbool;
@@ -445,6 +725,7 @@ ber_first_element(
 		*last = NULL;
 		return LBER_DEFAULT;
 	}
+	ber->ber_tag = *(unsigned char *)ber->ber_ptr;
 
 	*last = ber->ber_ptr + *len;
 
@@ -465,9 +746,9 @@ ber_next_element(
 	assert( len != NULL );
 	assert( last != NULL );
 
-	assert( BER_VALID( ber ) );
+	assert( LBER_VALID( ber ) );
 
-	if ( ber->ber_ptr == last ) {
+	if ( ber->ber_ptr >= last ) {
 		return LBER_DEFAULT;
 	}
 
@@ -482,14 +763,12 @@ ber_scanf ( BerElement *ber,
 {
 	va_list		ap;
 	LDAP_CONST char		*fmt_reset;
-	char		*last;
-	char		*s, **ss, ***sss;
-	struct berval 	***bv, **bvp, *bval;
+	char		*s, **ss;
+	struct berval	**bvp, *bval;
 	ber_int_t	*i;
-	int j;
 	ber_len_t	*l;
 	ber_tag_t	*t;
-	ber_tag_t	rc, tag;
+	ber_tag_t	rc;
 	ber_len_t	len;
 
 	va_start( ap, fmt );
@@ -497,13 +776,15 @@ ber_scanf ( BerElement *ber,
 	assert( ber != NULL );
 	assert( fmt != NULL );
 
-	assert( BER_VALID( ber ) );
+	assert( LBER_VALID( ber ) );
 
 	fmt_reset = fmt;
 
-	ber_log_printf( LDAP_DEBUG_TRACE, ber->ber_debug,
-		"ber_scanf fmt (%s) ber:\n", fmt );
-	ber_log_dump( LDAP_DEBUG_BER, ber->ber_debug, ber, 1 );
+	if ( ber->ber_debug & (LDAP_DEBUG_TRACE|LDAP_DEBUG_BER)) {
+		ber_log_printf( LDAP_DEBUG_TRACE, ber->ber_debug,
+			"ber_scanf fmt (%s) ber:\n", fmt );
+		ber_log_dump( LDAP_DEBUG_BER, ber->ber_debug, ber, 1 );
+	}
 
 	for ( rc = 0; *fmt && rc != LBER_DEFAULT; fmt++ ) {
 		/* When this is modified, remember to update
@@ -524,11 +805,23 @@ ber_scanf ( BerElement *ber,
 			rc = ber_get_stringa( ber, ss );
 			break;
 
+		case 'A':	/* octet string - allocate storage as needed,
+				 * but return NULL if len == 0 */
+			ss = va_arg( ap, char ** );
+			rc = ber_get_stringa_null( ber, ss );
+			break;
+
 		case 'b':	/* boolean */
 			i = va_arg( ap, ber_int_t * );
 			rc = ber_get_boolean( ber, i );
 			break;
 
+		case 'B':	/* bit string - allocate storage as needed */
+			ss = va_arg( ap, char ** );
+			l = va_arg( ap, ber_len_t * ); /* for length, in bits */
+			rc = ber_get_bitstringa( ber, ss, l );
+			break;
+
 		case 'e':	/* enumerated */
 		case 'i':	/* int */
 			i = va_arg( ap, ber_int_t * );
@@ -540,20 +833,35 @@ ber_scanf ( BerElement *ber,
 			rc = ber_peek_tag( ber, l );
 			break;
 
+		case 'm':	/* octet string in berval, in-place */
+			bval = va_arg( ap, struct berval * );
+			rc = ber_get_stringbv( ber, bval, 0 );
+			break;
+
+		case 'M':	/* bvoffarray - must include address of
+				 * a record len, and record offset.
+				 * number of records will be returned thru
+				 * len ptr on finish. parsed in-place.
+				 */
+		{
+			bgbvr cookie = { BvOff };
+			cookie.ber = ber;
+			cookie.res.ba = va_arg( ap, struct berval ** );
+			cookie.alloc = 0;
+			l = va_arg( ap, ber_len_t * );
+			cookie.siz = *l;
+			cookie.off = va_arg( ap, ber_len_t );
+			rc = ber_get_stringbvl( &cookie, l );
+			break;
+		}
+
 		case 'n':	/* null */
 			rc = ber_get_null( ber );
 			break;
 
-		case 's':	/* octet string - in a buffer */
-			s = va_arg( ap, char * );
-			l = va_arg( ap, ber_len_t * );
-			rc = ber_get_stringb( ber, s, l );
-			break;
-
 		case 'o':	/* octet string in a supplied berval */
 			bval = va_arg( ap, struct berval * );
-			ber_peek_tag( ber, &bval->bv_len );
-			rc = ber_get_stringa( ber, &bval->bv_val );
+			rc = ber_get_stringbv( ber, bval, LBER_BV_ALLOC );
 			break;
 
 		case 'O':	/* octet string - allocate & include length */
@@ -561,10 +869,10 @@ ber_scanf ( BerElement *ber,
 			rc = ber_get_stringal( ber, bvp );
 			break;
 
-		case 'B':	/* bit string - allocate storage as needed */
-			ss = va_arg( ap, char ** );
-			l = va_arg( ap, ber_len_t * ); /* for length, in bits */
-			rc = ber_get_bitstringa( ber, ss, l );
+		case 's':	/* octet string - in a buffer */
+			s = va_arg( ap, char * );
+			l = va_arg( ap, ber_len_t * );
+			rc = ber_get_stringb( ber, s, l );
 			break;
 
 		case 't':	/* tag of next item */
@@ -578,66 +886,46 @@ ber_scanf ( BerElement *ber,
 			break;
 
 		case 'v':	/* sequence of strings */
-			sss = va_arg( ap, char *** );
-			*sss = NULL;
-			j = 0;
-			for ( tag = ber_first_element( ber, &len, &last );
-			    tag != LBER_DEFAULT && rc != LBER_DEFAULT;
-			    tag = ber_next_element( ber, &len, last ) )
-			{
-				char **save = *sss;
-
-				*sss = (char **) LBER_REALLOC( *sss,
-					(j + 2) * sizeof(char *) );
-
-				if( *sss == NULL ) {
-					save[j] = NULL;
-					ber_memvfree( (void **) save );
-					rc = LBER_DEFAULT;
-					goto breakout;
-				}
-
-				rc = ber_get_stringa( ber, &((*sss)[j]) );
-				j++;
-			}
-			if ( j > 0 ) (*sss)[j] = NULL;
+		{
+			bgbvr cookie = { ChArray };
+			cookie.ber = ber;
+			cookie.res.c = va_arg( ap, char *** );
+			cookie.alloc = LBER_BV_ALLOC;
+			rc = ber_get_stringbvl( &cookie, NULL );
 			break;
+		}
 
 		case 'V':	/* sequence of strings + lengths */
-			bv = va_arg( ap, struct berval *** );
-			*bv = NULL;
-			j = 0;
-			for ( tag = ber_first_element( ber, &len, &last );
-			    tag != LBER_DEFAULT && rc != LBER_DEFAULT;
-			    tag = ber_next_element( ber, &len, last ) )
-			{
-				struct berval **save = *bv;
-
-				*bv = (struct berval **) LBER_REALLOC( *bv,
-					(j + 2) * sizeof(struct berval *) );
-		
-				if( *bv == NULL ) {
-					save[j] = NULL;
-					ber_bvecfree( save );
-					rc = LBER_DEFAULT;
-					goto breakout;
-				}
-
-				rc = ber_get_stringal( ber, &((*bv)[j]) );
-				j++;
-			}
-			if ( j > 0 ) (*bv)[j] = NULL;
+		{
+			bgbvr cookie = { BvVec };
+			cookie.ber = ber;
+			cookie.res.bv = va_arg( ap, struct berval *** );
+			cookie.alloc = LBER_BV_ALLOC;
+			rc = ber_get_stringbvl( &cookie, NULL );
 			break;
+		}
+
+		case 'W':	/* bvarray */
+		{
+			bgbvr cookie = { BvArray };
+			cookie.ber = ber;
+			cookie.res.ba = va_arg( ap, struct berval ** );
+			cookie.alloc = LBER_BV_ALLOC;
+			rc = ber_get_stringbvl( &cookie, NULL );
+			break;
+		}
 
 		case 'x':	/* skip the next element - whatever it is */
 			if ( (rc = ber_skip_tag( ber, &len )) == LBER_DEFAULT )
 				break;
 			ber->ber_ptr += len;
+			ber->ber_tag = *(unsigned char *)ber->ber_ptr;
 			break;
 
 		case '{':	/* begin sequence */
 		case '[':	/* begin set */
-			if ( *(fmt + 1) != 'v' && *(fmt + 1) != 'V' )
+			if ( *(fmt + 1) != 'v' && *(fmt + 1) != 'V'
+				&& *(fmt + 1) != 'W' && *(fmt + 1) != 'M' )
 				rc = ber_skip_tag( ber, &len );
 			break;
 
@@ -655,17 +943,15 @@ ber_scanf ( BerElement *ber,
 		}
 	}
 
-breakout:
 	va_end( ap );
-
 	if ( rc == LBER_DEFAULT ) {
-	    /*
-	     * Error.  Reclaim malloced memory that was given to the caller.
-	     * Set allocated pointers to NULL, "data length" outvalues to 0.
-	     */
-	    va_start( ap, fmt );
+		/*
+		 * Error.  Reclaim malloced memory that was given to the caller.
+		 * Set allocated pointers to NULL, "data length" outvalues to 0.
+		 */
+		va_start( ap, fmt );
 
-	    for ( ; fmt_reset < fmt; fmt_reset++ ) {
+		for ( ; fmt_reset < fmt; fmt_reset++ ) {
 		switch ( *fmt_reset ) {
 		case '!': { /* Hook */
 				BERDecodeCallback *f;
@@ -678,6 +964,7 @@ breakout:
 			} break;
 
 		case 'a':	/* octet string - allocate storage as needed */
+		case 'A':
 			ss = va_arg( ap, char ** );
 			if ( *ss ) {
 				LBER_FREE( *ss );
@@ -691,20 +978,10 @@ breakout:
 			(void) va_arg( ap, int * );
 			break;
 
-		case 's':	/* octet string - in a buffer */
-			(void) va_arg( ap, char * );
-			(void) va_arg( ap, ber_len_t * );
-			break;
-
 		case 'l':	/* length of next item */
 			(void) va_arg( ap, ber_len_t * );
 			break;
 
-		case 't':	/* tag of next item */
-		case 'T':	/* skip tag of next item */
-			(void) va_arg( ap, ber_tag_t * );
-			break;
-
 		case 'o':	/* octet string in a supplied berval */
 			bval = va_arg( ap, struct berval * );
 			if ( bval->bv_val != NULL ) {
@@ -722,6 +999,16 @@ breakout:
 			}
 			break;
 
+		case 's':	/* octet string - in a buffer */
+			(void) va_arg( ap, char * );
+			(void) va_arg( ap, ber_len_t * );
+			break;
+
+		case 't':	/* tag of next item */
+		case 'T':	/* skip tag of next item */
+			(void) va_arg( ap, ber_tag_t * );
+			break;
+
 		case 'B':	/* bit string - allocate storage as needed */
 			ss = va_arg( ap, char ** );
 			if ( *ss ) {
@@ -731,23 +1018,12 @@ breakout:
 			*(va_arg( ap, ber_len_t * )) = 0; /* for length, in bits */
 			break;
 
-		case 'v':	/* sequence of strings */
-			sss = va_arg( ap, char *** );
-			if ( *sss ) {
-				ber_memvfree( (void **) *sss );
-				*sss = NULL;
-			}
-			break;
-
-		case 'V':	/* sequence of strings + lengths */
-			bv = va_arg( ap, struct berval *** );
-			if ( *bv ) {
-				ber_bvecfree( *bv );
-				*bv = NULL;
-			}
-			break;
-
+		case 'm':	/* berval in-place */
+		case 'M':	/* BVoff array in-place */
 		case 'n':	/* null */
+		case 'v':	/* sequence of strings */
+		case 'V':	/* sequence of strings + lengths */
+		case 'W':	/* BerVarray */
 		case 'x':	/* skip the next element - whatever it is */
 		case '{':	/* begin sequence */
 		case '[':	/* begin set */
@@ -759,9 +1035,9 @@ breakout:
 			/* format should be good */
 			assert( 0 );
 		}
-	    }
+		}
 
-	    va_end( ap );
+		va_end( ap );
 	}
 
 	return rc;
