@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/libraries/libldap/dnssrv.c,v 1.26.2.8 2005/01/20 17:01:01 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/libldap/dnssrv.c,v 1.39.2.4 2008/02/11 23:26:41 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2005 The OpenLDAP Foundation.
+ * Copyright 1998-2008 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -69,7 +69,7 @@ int ldap_dn2domain(
 				(ava->la_flags & LDAP_AVA_STRING) &&
 				ava->la_value.bv_len &&
 				( ber_bvstrcasecmp( &ava->la_attr, &DC ) == 0
-				|| ber_bvstrcasecmp( &ava->la_attr, &DCOID ) == 0 ) )
+				|| ber_bvcmp( &ava->la_attr, &DCOID ) == 0 ) )
 			{
 				if( domain.bv_len == 0 ) {
 					ndomain = LDAP_REALLOC( domain.bv_val,
@@ -275,8 +275,12 @@ int ldap_domain2hostlist(
 		/* weight = (p[2] << 8) | p[3]; */
 		port = (p[4] << 8) | p[5];
 
-		buflen = strlen(host) + sizeof(":65355 ");
-		hostlist = (char *) LDAP_REALLOC(hostlist, cur + buflen);
+		if ( port == 0 || host[ 0 ] == '\0' ) {
+		    goto add_size;
+		}
+
+		buflen = strlen(host) + STRLENOF(":65355 ");
+		hostlist = (char *) LDAP_REALLOC(hostlist, cur + buflen + 1);
 		if (hostlist == NULL) {
 		    rc = LDAP_NO_MEMORY;
 		    goto out;
@@ -287,6 +291,7 @@ int ldap_domain2hostlist(
 		}
 		cur += sprintf(&hostlist[cur], "%s:%hd", host, port);
 	    }
+add_size:;
 	    p += size;
 	}
     }
