@@ -1,8 +1,8 @@
 /* tls.c - Handle tls/ssl using SSLeay, OpenSSL or GNUTLS. */
-/* $OpenLDAP: pkg/ldap/libraries/libldap/tls.c,v 1.133.2.7 2007/08/31 23:13:56 quanah Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/libldap/tls.c,v 1.133.2.11 2008/07/09 23:56:44 quanah Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2007 The OpenLDAP Foundation.
+ * Copyright 1998-2008 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -307,6 +307,7 @@ tls_parse_ciphers( tls_ctx *ctx, char *suites )
 			/* unrecognized cipher suite */
 			return -1;
 		}
+		ptr += len + 1;
 	} while (end);
 
 	/* Space for all 3 lists */
@@ -348,6 +349,7 @@ tls_parse_ciphers( tls_ctx *ctx, char *suites )
 				break;
 			}
 		}
+		ptr += len + 1;
 	} while (end);
 	kx[nkx] = 0;
 	cipher[ncipher] = 0;
@@ -753,6 +755,7 @@ ldap_int_tls_init_ctx( struct ldapoptions *lo, int is_server )
 			crlfile,
 			GNUTLS_X509_FMT_PEM );
 		if ( rc < 0 ) goto error_exit;
+		rc = 0;
 	}
 	if ( is_server ) {
 		gnutls_dh_params_init (&((tls_ctx*) 
@@ -2792,7 +2795,7 @@ ldap_pvt_tls_get_strength( void *s )
 	gnutls_cipher_algorithm_t c;
 
 	c = gnutls_cipher_get( session->session );
-	return gnutls_cipher_get_key_size( c );
+	return gnutls_cipher_get_key_size( c ) * 8;
 #else
 	return 0;
 #endif
